"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PropertyScalarBuilder = void 0;
const simple_value_provider_1 = require("../../provider/value-provider/simple-value-provider");
const select_value_converter_1 = require("../../value-converter/choices/select-value-converter");
const choice_value_provider_1 = require("../../provider/value-provider/choices/choice-value-provider");
const object_value_provider_1 = require("../../provider/value-provider/object-value-provider");
const common_value_converters_1 = require("../../value-converter/common-value-converters");
const derived_value_provider_1 = require("../../provider/value-provider/derived-value-provider");
const derived_async_value_provider_1 = require("../../provider/value-provider/derived-async-value-provider");
const constant_value_provider_1 = require("../../provider/value-provider/constant-value-provider");
const empty_value_fcn_1 = require("../../provider/value-provider/empty-value-fcn");
const choice_list_converter_1 = require("../../value-converter/choices/choice-list-converter");
const assertions_1 = require("../../util/assertions/assertions");
const select_value_provider_1 = require("../../provider/value-provider/choices/select-value-provider");
const typeahead_value_provider_1 = require("../../provider/value-provider/choices/typeahead-value-provider");
const typeahead_value_converter_1 = require("../../value-converter/choices/typeahead-value-converter");
const property_scalar_with_choices_1 = require("../../properties/property-scalar-with-choices");
class PropertyScalarBuilder {
    constructor(propertyScalar, bindPropertScalar, defaultEmptyChoice) {
        this.propertyScalar = propertyScalar;
        this.bindPropertScalar = bindPropertScalar;
        this.defaultEmptyChoice = defaultEmptyChoice;
        this.select = {
            static: (id, choices, emptyChoice) => {
                var _a;
                if (!emptyChoice) {
                    emptyChoice = this.defaultEmptyChoice;
                }
                const provider = new choice_value_provider_1.ChoiceValueProvider(choices, emptyChoice);
                const converter = new select_value_converter_1.SelectValueConverter(choices, emptyChoice);
                const emptyValueFcn = emptyChoice ? empty_value_fcn_1.EmptyValueFcns.choiceEmptyValueFcn(emptyChoice) : empty_value_fcn_1.EmptyValueFcns.defaultEmptyValueFcn;
                const prop = this.propertyScalar(id, provider, emptyValueFcn, converter);
                prop.defineInitialValue((emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value) !== undefined ? emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value : (_a = choices[0]) === null || _a === void 0 ? void 0 : _a.value);
                prop.setToInitialValue();
                return property_scalar_with_choices_1.upgradeAsPropertyWithChoices(prop, () => provider.getChoices());
            },
            derived: (id, ...dependencies) => (derivations, emptyChoice) => {
                const choicesSourceProperty = this.derived.sync(`${id}__choices__`, new choice_list_converter_1.ChoiceListConverter(), ...dependencies)(derivations);
                return this.select.withPropertySource(id, choicesSourceProperty, emptyChoice);
            },
            asyncDerived: (id, ...dependencies) => (derivations, emptyChoice) => {
                const choicesSourceProperty = this.derived.async(`${id}__choices__`, new choice_list_converter_1.ChoiceListConverter(), ...dependencies)(derivations);
                return this.select.withPropertySource(id, choicesSourceProperty, emptyChoice);
            },
            withPropertySource: (id, choicesSource, emptyChoice) => {
                var _a;
                if (!emptyChoice) {
                    emptyChoice = this.defaultEmptyChoice;
                }
                const provider = new select_value_provider_1.SelectValueProvider(choicesSource, emptyChoice);
                const converter = new select_value_converter_1.SelectValueConverter(() => choicesSource.getNonNullValue(), emptyChoice);
                const emptyValueFcn = emptyChoice ? empty_value_fcn_1.EmptyValueFcns.choiceEmptyValueFcn(emptyChoice) : empty_value_fcn_1.EmptyValueFcns.defaultEmptyValueFcn;
                const prop = this.propertyScalar(id, provider, emptyValueFcn, converter, [choicesSource], undefined, undefined, [choicesSource]);
                prop.defineInitialValue((emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value) !== undefined ? emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value : (_a = choicesSource.getNonNullValue()[0]) === null || _a === void 0 ? void 0 : _a.value);
                prop.setToInitialValue();
                return property_scalar_with_choices_1.upgradeAsPropertyWithChoices(prop, () => provider.getChoices());
            }
        };
        this.typeahead = {
            async: (id, options) => {
                const inputSourceProperty = this.stringProperty(`${id}__input__`);
                const choicesSourceProperty = this.derived.async(`${id}__choices__`, new choice_list_converter_1.ChoiceListConverter(), inputSourceProperty)({
                    deriveAsync: (inputProperty) => {
                        const text = inputProperty.getNonNullValue();
                        if (options.minimumTextLength && text.length < options.minimumTextLength) {
                            return Promise.resolve([]);
                        }
                        return options.fetchChoices(text);
                    }
                });
                return this.typeahead.withPropertySource(id, inputSourceProperty, choicesSourceProperty);
            },
            withPropertySource: (id, inputSource, choicesSource) => {
                const provider = new typeahead_value_provider_1.TypeaheadValueProvider(inputSource, choicesSource);
                const converter = new typeahead_value_converter_1.TypeaheadValueConverter();
                const emptyValueFcn = (val) => (val === null || val === void 0 ? void 0 : val[0]) == null;
                const prop = this.propertyScalar(id, provider, emptyValueFcn, converter, [inputSource], undefined, undefined, [inputSource, choicesSource]);
                return property_scalar_with_choices_1.upgradeAsPropertyWithChoices(prop, choicesSource);
            }
        };
        this.derived = {
            sync: (id, valueConverter, ...dependencies) => {
                return (derivations, emptyValueFcn = empty_value_fcn_1.EmptyValueFcns.defaultEmptyValueFcn) => {
                    const provider = new derived_value_provider_1.DerivedValueProvider(dependencies, (deps) => derivations.derive(...deps), derivations === null || derivations === void 0 ? void 0 : derivations.inverse);
                    return this.propertyScalar(id, provider, emptyValueFcn, valueConverter, dependencies);
                };
            },
            async: (id, valueConverter, ...dependencies) => {
                return (derivations, emptyValueFcn = empty_value_fcn_1.EmptyValueFcns.defaultEmptyValueFcn) => {
                    const provider = new derived_async_value_provider_1.DerivedAsyncValueProvider(dependencies, (deps) => derivations.deriveAsync(...deps), derivations === null || derivations === void 0 ? void 0 : derivations.inverseAsync);
                    return this.propertyScalar(id, provider, emptyValueFcn, valueConverter, dependencies, undefined, derivations.backpressureConfig);
                };
            }
        };
        assertions_1.assertThat(() => !defaultEmptyChoice || defaultEmptyChoice.value === null, () => 'value of defaultEmptyChoice must be null to match every Choice<T>');
    }
    template(id, factory) {
        return (prefix, index, siblingAccess) => factory(this, `${prefix}_${id}`, index, siblingAccess);
    }
    isLike(template, valueProvider) {
        return template;
    }
    simpleProperty(id, valueConverter, emptyValueFcn = empty_value_fcn_1.EmptyValueFcns.defaultEmptyValueFcn) {
        return this.propertyScalar(id, new simple_value_provider_1.SimpleValueProvider(), emptyValueFcn, valueConverter);
    }
    stringProperty(id, options) {
        var _a;
        return this.propertyScalar(id, new simple_value_provider_1.SimpleValueProvider(), empty_value_fcn_1.EmptyValueFcns.defaultEmptyValueFcn, (_a = options === null || options === void 0 ? void 0 : options.valueConverter) !== null && _a !== void 0 ? _a : common_value_converters_1.C.string.identity, undefined, (options === null || options === void 0 ? void 0 : options.initialValue) !== undefined ? options === null || options === void 0 ? void 0 : options.initialValue : '');
    }
    numberProperty(id, options) {
        var _a;
        return this.propertyScalar(id, new simple_value_provider_1.SimpleValueProvider(), (options === null || options === void 0 ? void 0 : options.zeroIsConsideredAsEmpty) ? empty_value_fcn_1.EmptyValueFcns.defaultEmptyValueFcn : empty_value_fcn_1.EmptyValueFcns.numberEmptyValueFcn, (_a = options === null || options === void 0 ? void 0 : options.valueConverter) !== null && _a !== void 0 ? _a : common_value_converters_1.C.number.default, undefined, (options === null || options === void 0 ? void 0 : options.initialValue) !== undefined ? options === null || options === void 0 ? void 0 : options.initialValue : ((options === null || options === void 0 ? void 0 : options.zeroIsConsideredAsEmpty) ? 0 : null));
    }
    booleanProperty(id, options) {
        var _a;
        return this.propertyScalar(id, new simple_value_provider_1.SimpleValueProvider(), empty_value_fcn_1.EmptyValueFcns.booleanEmptyValueFcn, (_a = options === null || options === void 0 ? void 0 : options.valueConverter) !== null && _a !== void 0 ? _a : common_value_converters_1.C.boolean.default, undefined, options === null || options === void 0 ? void 0 : options.initialValue);
    }
    dateProperty(id, options) {
        var _a;
        return this.propertyScalar(id, new simple_value_provider_1.SimpleValueProvider(), empty_value_fcn_1.EmptyValueFcns.defaultEmptyValueFcn, (_a = options === null || options === void 0 ? void 0 : options.valueConverter) !== null && _a !== void 0 ? _a : common_value_converters_1.C.date.iso, undefined, options === null || options === void 0 ? void 0 : options.initialValue);
    }
    objectProperty(id, obj, get, set, valueConverter, emptyValueFcn = empty_value_fcn_1.EmptyValueFcns.defaultEmptyValueFcn) {
        return this.propertyScalar(id, new object_value_provider_1.ObjectValueProvider(obj, get, set), emptyValueFcn, valueConverter);
    }
    objectPathProperty(id, obj, path, valueConverter, emptyValueFcn = empty_value_fcn_1.EmptyValueFcns.defaultEmptyValueFcn) {
        const get = (obj) => null;
        const set = (obj, val) => {
        };
        return this.propertyScalar(id, new object_value_provider_1.ObjectValueProvider(obj, get, set), emptyValueFcn, valueConverter);
    }
    enviromentProperty(id, valueConverter, value, emptyValueFcn = empty_value_fcn_1.EmptyValueFcns.defaultEmptyValueFcn) {
        let provider;
        if (value instanceof Function) {
            provider = new derived_value_provider_1.DerivedValueProvider([], (deps) => value());
        }
        else {
            provider = new constant_value_provider_1.ConstantValueProvider(value);
        }
        return this.propertyScalar(id, provider, emptyValueFcn, valueConverter);
    }
    bind(prop) {
        return this.bindPropertScalar(prop);
    }
}
exports.PropertyScalarBuilder = PropertyScalarBuilder;
