import { PropertyId } from "../../properties/property-id";
import { PropertyScalar } from "../../properties/property-scalar";
import { PropertyScalarImpl } from "../../properties/property-scalar-impl";
import { AbstractProperty } from "../../properties/abstract-property";
import { ValueConverter } from "../../value-converter/value-converter";
import { Choice } from "../../properties/choice";
import { ValueProvider } from "../../provider/value-provider/value-provider";
import { PropertyScalarRuleBinding } from "./property-scalar-rule-binding";
import { PropertyTemplate } from "../../properties/factory/property-template";
import { EmptyValueFcn } from "../../provider/value-provider/empty-value-fcn";
import { BackpressureConfig } from "../../properties/backpressure/backpressure-config";
import { PropertyScalarWithChoices } from "../../properties/property-scalar-with-choices";
import { ListIndex } from "../../properties/factory/list-index";
import { SiblingAccess } from "../../provider/list-provider/sibling-access";
import { Rule } from "../../rules/rule";
export declare class PropertyScalarBuilder {
    private readonly propertyScalar;
    private readonly bindPropertScalar;
    private readonly defaultEmptyChoice;
    constructor(propertyScalar: <T>(id: PropertyId, provider: ValueProvider<T>, emptyValueFcn: EmptyValueFcn<T>, converter: ValueConverter<T>, dependencies?: readonly AbstractProperty<unknown>[], initialValue?: T | null, backpressureConfig?: BackpressureConfig, ownedProperties?: readonly AbstractProperty<unknown>[]) => PropertyScalarImpl<T>, bindPropertScalar: <T>(prop: PropertyScalar<T>) => PropertyScalarRuleBinding<T>, defaultEmptyChoice: Choice<any> | undefined);
    template<T>(id: string, factory: (scalarBuilder: PropertyScalarBuilder, id: PropertyId, index?: ListIndex, siblingAccess?: SiblingAccess<PropertyScalar<T>, T>) => PropertyScalar<T>): PropertyTemplate<PropertyScalar<T>, T>;
    isLike<T>(template: PropertyScalar<T>, valueProvider?: ValueProvider<T>): PropertyScalar<T>;
    simpleProperty<T>(id: PropertyId, valueConverter: ValueConverter<T>, emptyValueFcn?: EmptyValueFcn<T>): PropertyScalar<T>;
    stringProperty(id: PropertyId, options?: {
        valueConverter?: ValueConverter<string>;
        initialValue?: string | null;
    }): PropertyScalar<string>;
    numberProperty(id: PropertyId, options?: {
        valueConverter?: ValueConverter<number>;
        zeroIsConsideredAsEmpty?: boolean;
        initialValue?: number | null;
    }): PropertyScalar<number>;
    booleanProperty(id: PropertyId, options?: {
        valueConverter?: ValueConverter<boolean>;
        initialValue?: boolean | null;
    }): PropertyScalar<boolean>;
    dateProperty(id: PropertyId, options?: {
        valueConverter?: ValueConverter<Date>;
        initialValue?: Date | null;
    }): PropertyScalar<Date>;
    select: {
        static: <T>(id: PropertyId, choices: Choice<T>[], emptyChoice?: Choice<T> | undefined) => PropertyScalarWithChoices<T, T>;
        derived: <T_1, Dependencies extends readonly AbstractProperty<unknown>[]>(id: PropertyId, ...dependencies: Dependencies) => (derivations: {
            derive: (...dependencies: Dependencies) => Choice<T_1>[];
        }, emptyChoice?: Choice<T_1> | undefined) => PropertyScalarWithChoices<T_1, T_1>;
        asyncDerived: <T_2, Dependencies_1 extends readonly AbstractProperty<unknown>[]>(id: PropertyId, ...dependencies: Dependencies_1) => (derivations: {
            deriveAsync: (...dependencies: Dependencies_1) => Promise<Choice<T_2>[]>;
            backpressureConfig?: BackpressureConfig | undefined;
        }, emptyChoice?: Choice<T_2> | undefined) => PropertyScalarWithChoices<T_2, T_2>;
        withPropertySource: <T_3>(id: PropertyId, choicesSource: PropertyScalar<Choice<T_3>[]>, emptyChoice?: Choice<T_3> | undefined) => PropertyScalarWithChoices<T_3, T_3>;
    };
    typeahead: {
        async: <T>(id: PropertyId, options: {
            fetchChoices: (currentText: string) => Promise<Choice<T>[]>;
            minimumTextLength?: number | undefined;
        }) => PropertyScalarWithChoices<[T | null, string], T>;
        withPropertySource: <T_1>(id: PropertyId, inputSource: PropertyScalar<string>, choicesSource: PropertyScalar<Choice<T_1>[]>) => PropertyScalarWithChoices<[T_1 | null, string], T_1>;
    };
    objectProperty<T, O>(id: PropertyId, obj: O, get: (obj: O) => T | null, set: (obj: O, val: T | null) => void, valueConverter: ValueConverter<T>, emptyValueFcn?: EmptyValueFcn<T>): PropertyScalar<T>;
    objectPathProperty<T, O>(id: PropertyId, obj: O, path: string, valueConverter: ValueConverter<T>, emptyValueFcn?: EmptyValueFcn<T>): PropertyScalar<T>;
    enviromentProperty<T>(id: PropertyId, valueConverter: ValueConverter<T>, value: T | (() => T), emptyValueFcn?: EmptyValueFcn<T>): PropertyScalar<T>;
    derived: {
        sync: <T, Dependencies extends readonly AbstractProperty<unknown>[]>(id: PropertyId, valueConverter: ValueConverter<T>, ...dependencies: Dependencies) => (derivations: {
            derive: Rule<[...dependencies: Dependencies], T | null>;
            inverse?: ((val: T | null, ...dependencies: Dependencies) => void) | undefined;
        }, emptyValueFcn?: EmptyValueFcn<T>) => PropertyScalar<T>;
        async: <T_1, Dependencies_1 extends readonly AbstractProperty<unknown>[]>(id: PropertyId, valueConverter: ValueConverter<T_1>, ...dependencies: Dependencies_1) => (derivations: {
            deriveAsync: Rule<[...dependencies: Dependencies_1], Promise<T_1 | null>>;
            inverseAsync?: ((val: T_1 | null, ...dependencies: Dependencies_1) => Promise<void>) | undefined;
            backpressureConfig?: BackpressureConfig | undefined;
        }, emptyValueFcn?: EmptyValueFcn<T_1>) => PropertyScalar<T_1>;
    };
    bind<T>(prop: PropertyScalar<T>): PropertyScalarRuleBinding<T>;
}
